#include "ostdb.h"
#include "error.h"
#include "dbquery.h"
#include "config.h"
#include "loadfile.h"
#include "cdbslib.h"
#include "getref.h"

#define  KEY_FIELD	"ocf_keyword"
#define  BEST_FIELD	"ocf_best_fieldname"
#define  OLD_FIELD	"ocf_old_fieldname"

/* GET_KEYWORDS -- Retrieve keywords and their associated database fields
 *
 * i: qry		database query context
 * i: dataset		dataset keywords are asked for
 * i: file_type		type of file requested: "science" or "wavecal"
 * i: field_type	type of field requested: "switch" or "ref_file"
 * o: keyword		array of header keyword names
 * o: bestfield		array of db fields containing best reference files
 * o: oldfield		array of db fields containing old reference files
 * i: maxfield		max length of output arrays
 */

int get_keywords (context *qry, char *dataset, char *file_type, 
		  char *field_type, char **keyword, char **bestfield, 
		  char **oldfield, int maxfield) {

    int		code, ifield, status;
    char	ty_file[SZ_STRING+1], ty_field[SZ_STRING+1], inst[SZ_STRING+1];
    char	keyval[SZ_STRING+1], bestval[SZ_STRING+1], oldval[SZ_STRING+1];

    static char *fileqry_fmt = "select ocf_keyword, ocf_best_fieldname, "
	"ocf_old_fieldname from otfc_calibration_fields where "
	"ocf_instrument = ^ and ocf_file_type = ^ and ocf_field_type = ^ "
	"order by ocf_sequence_number";

    /* Check file and keyword types for legal values */

    if (strmatch (file_type, FILE_TYPE) == 0)
	cdbs_error (NOMATCH, "in get_keywords: file_type = %s", file_type);

    if (strmatch (field_type, FIELD_TYPE) == 0)
	cdbs_error (NOMATCH, "in get_keywords: field_type = %s", field_type);

    /* Get instrument code and keyword type from the inputs */

    /* Set up query to retrieve keyword names and database fields */

    code = get_icode_obs (dataset);

    ostdb_query_setup (qry->connection, "get_keywords", 1, STDB_SQL,
		       qry->server, qry->dads_db, db_error);

    stdb_bind_string (qry->connection, KEY_FIELD, keyval, SZ_STRING);
    stdb_bind_string (qry->connection, BEST_FIELD, bestval, SZ_STRING);
    stdb_bind_string (qry->connection, OLD_FIELD, oldval, SZ_STRING);

    safecopy (inst, strucase (enquote (inst_dads[code])), SZ_STRING);
    safecopy (ty_file, strucase (enquote (file_type)), SZ_STRING);
    safecopy (ty_field, strucase (enquote (field_type)), SZ_STRING);

    build_qry (qry->connection, fileqry_fmt, inst, ty_file, ty_field);

    /* Retrieve info and store in output arrays */

    ifield = 0;
    while (ostdb_get_exec (qry->connection) == STDB_CONTINUE && 
	   ifield < maxfield) {

	clean_data (keyval);
	clean_data (bestval);
	clean_data (oldval);

	/* If the keyword name is blank, that means the field is not
	 * yet in the keyword database and should not be used
	 */

	if (keyval[0] != EOS) {
	    keyword[ifield] = strclone (keyval);
	    bestfield[ifield] = strclone (bestval);
	    oldfield[ifield] = strclone (oldval);

	    ifield ++;
	}
    }

    /* Terminate arrays with zero length strings to mark end of array */

    if (ifield >= maxfield) {
        status = ERR;
	cdbs_error (OVFLOW, "%s", "get_keywords");

    } else if (ifield == 0) {
        status = ERR;

    } else {
        status = OK;
	keyword[ifield] = strclone ("");
	bestfield[ifield] = strclone ("");
	oldfield[ifield] = strclone ("");
    }

    stdb_close_query (qry->connection);
    return (status);
}
